from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.urls import re_path

__all__ = ("urlpatterns",)

urlpatterns = [
    re_path(r"^admin/", admin.site.urls),
]

# Serving media and static in debug/developer mode.
if settings.DEBUG:
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from typing import Optional

import pdfkit
from faker.providers import BaseProvider

from ..base import FileMixin, StringValue
from ..constants import DEFAULT_FILE_ENCODING, DEFAULT_TEXT_MAX_NB_CHARS
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("PdfFileProvider",)


class PdfFileProvider(BaseProvider, FileMixin):
    """PDF file provider.

        Usage example:

        from faker_file.providers.pdf_file import PdfFileProvider

        file = PdfFileProvider(None).pdf_file()

    Usage example with options:

        from faker_file.providers.pdf_file import PdfFileProvider

        file = PdfFileProvider(None).pdf_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = PdfFileProvider(Faker()).pdf_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )
    """

    extension: str = "pdf"

    def pdf_file(
        self: "PdfFileProvider",
        storage: BaseStorage = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        encoding: Optional[str] = DEFAULT_FILE_ENCODING,
        **kwargs,
    ) -> StringValue:
        """Generate a PDF file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param encoding: Encoding of the file.
        :return: Relative path (from root directory) of the generated file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        content = self._generate_text_content(
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
        )

        options = {"quiet": ""}
        if encoding is not None:
            options["encoding"] = encoding

        raw_content = pdfkit.from_string(
            f"<pre>{content}</pre>",
            options=options,
        )

        storage.write_bytes(filename, raw_content)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = {"content": content}
        return file_name



# import mimetypes
import os
from email.message import EmailMessage
from email.policy import default
from typing import Any, Dict, Optional

from faker.providers import BaseProvider

from ..base import FileMixin, StringValue
from ..constants import DEFAULT_TEXT_MAX_NB_CHARS
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage
from .helpers.inner import create_inner_txt_file

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("EmlFileProvider",)


class EmlFileProvider(BaseProvider, FileMixin):
    """EML file provider.

    Usage example:

        from faker import Faker
        from faker_file.providers.eml_file import EmlFileProvider

        FAKER = Faker()

        file = EmlFileProvider(FAKER).eml_file()

    Usage example with attachments:

        from faker_file.providers.helpers.inner import create_inner_docx_file
        from faker_file.providers.eml_file import EmlFileProvider

        file = EmlFileProvider(FAKER).eml_file(
            prefix="zzz_email_",
            options={
                "count": 5,
                "create_inner_file_func": create_inner_docx_file,
                "create_inner_file_args": {
                    "prefix": "zzz_docx_file_",
                    "max_nb_chars": 1_024,
                },
            }
        )

    Usage example of nested EMLs attachments:

        from faker_file.providers.helpers.inner import create_inner_eml_file

        file = EmlFileProvider(FAKER).eml_file(
            options={
                "create_inner_file_func": create_inner_eml_file,
                "create_inner_file_args": {
                    "options": {
                        "create_inner_file_func": create_inner_docx_file,
                    }
                }
            }
        )

    If you want to see, which files were included inside the EML, check
    the ``file.data["files"]``.
    """

    extension: str = "eml"

    def eml_file(
        self: "EmlFileProvider",
        storage: BaseStorage = None,
        prefix: Optional[str] = None,
        options: Optional[Dict[str, Any]] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        """Generate an EML file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param options: Options (non-structured) for complex types, such as ZIP.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :return: Relative path (from root directory) of the generated file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )
        fs_storage = FileSystemStorage()

        content = self._generate_text_content(
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
        )
        data = {
            "content": content,
            "inner": {},
        }

        msg = EmailMessage()
        msg["To"] = self.generator.email()
        msg["From"] = self.generator.email()
        msg["Subject"] = self.generator.sentence()
        msg.set_content(content)
        data.update(
            {
                "to": msg["To"],
                "from": msg["From"],
                "subject": msg["Subject"],
                "content": content,
            }
        )

        # Specific
        if options:
            """
            A complex case. Could be initialized as follows:

                eml_file = EmlFileProvider(FAKER).eml_file(
                    prefix="zzz_email_",
                    options={
                        "count": 5,
                        "create_inner_file_func": create_inner_docx_file,
                        "create_inner_file_args": {
                            "prefix": "zzz_file_",
                            "max_nb_chars": 1_024,
                            "content": "{{date}}\r\n{{text}}\r\n{{name}}",
                        },
                    }
                )
            """
            _count = options.get("count", 5)
            _create_inner_file_func = options.get(
                "create_inner_file_func", create_inner_txt_file
            )
            _create_inner_file_args = options.get("create_inner_file_args", {})

        else:
            # Defaults
            _count = 0
            _create_inner_file_func = create_inner_txt_file
            _create_inner_file_args = {}

        _kwargs = {"generator": self.generator}
        _kwargs.update(_create_inner_file_args)

        for __i in range(_count):
            __file = _create_inner_file_func(
                storage=fs_storage,
                **_kwargs,
            )
            data["inner"][str(__file)] = __file
            __file_abs_path = fs_storage.abspath(__file)
            # _content_type, _encoding = mimetypes.guess_type(__file_abs_path)
            # if _content_type is None or _encoding is not None:
            #     # No guess could be made, or the file is
            #     # encoded (compressed), so use a generic bag-of-bits type.
            #     _content_type = "application/octet-stream"
            _content_type = "application/octet-stream"
            _maintype, _subtype = _content_type.split("/", 1)
            with open(__file_abs_path, "rb") as _fp:
                _file_data = _fp.read()
                msg.add_attachment(
                    _file_data,
                    maintype=_maintype,
                    subtype=_subtype,
                    filename=os.path.basename(__file),
                )
            os.remove(__file_abs_path)  # Clean up temporary files

        storage.write_bytes(filename, msg.as_bytes(policy=default))

        # Generic
        file_name = StringValue(storage.relpath(filename))
        if data:
            file_name.data = data
        return file_name



from typing import Optional

from faker.providers import BaseProvider

from ..base import FileMixin, StringValue
from ..constants import DEFAULT_TEXT_MAX_NB_CHARS
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("TxtFileProvider",)


class TxtFileProvider(BaseProvider, FileMixin):
    """TXT file provider.

    Usage example:

        from faker import Faker
        from faker_file.providers.txt_file import TxtFileProvider

        file = TxtFileProvider(Faker()).txt_file()

    Usage example with options:

        file = TxtFileProvider(Faker()).txt_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = TxtFileProvider(Faker()).txt_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )
    """

    extension: str = "txt"

    def txt_file(
        self: "TxtFileProvider",
        storage: BaseStorage = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        """Generate a TXT file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :return: Relative path (from root directory) of the generated file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        content = self._generate_text_content(
            max_nb_chars=max_nb_chars,
            wrap_chars_after=wrap_chars_after,
            content=content,
        )

        storage.write_text(filename, content)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = {"content": content}
        return file_name



from typing import Dict, List, Optional, Union

from faker import Faker
from tablib import Dataset

from ...base import FileMixin, StringValue
from ...storages.base import BaseStorage
from ...storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("TabularDataMixin",)


class TabularDataMixin(FileMixin):
    """Tabular data mixin."""

    def _tabular_data_file(
        self: "TabularDataMixin",
        storage: BaseStorage = None,
        prefix: Optional[str] = None,
        data_columns: Dict[str, str] = None,
        num_rows: int = 10,
        content: Optional[str] = None,
        **kwargs,
    ) -> StringValue:
        """Generate a tabular data file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param data_columns: The ``data_columns`` argument expects a list or a
            tuple of string tokens, and these string tokens will be passed to
            :meth:`pystr_format()
            <faker.providers.python.Provider.pystr_format>`
            for data generation. Argument Groups are used to pass arguments
            to the provider methods. Both ``header`` and ``data_columns`` must
            be of the same length.
        :param num_rows: The ``num_rows`` argument controls how many rows of
            data to generate, and the ``include_row_ids`` argument may be set
            to ``True`` to include a sequential row ID column.
        :param prefix: File name prefix.
        :param content: List of dicts with content (JSON-like format).
            If given, used as is.
        :return: Relative path (from root directory) of the generated file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            prefix=prefix,
            extension=self.extension,
        )

        if self.generator is None:
            self.generator = Faker()

        # Specific
        if content is None:
            default_data_columns = {
                "name": "{{name}}",
                "residency": "{{address}}",
            }
            data_columns: Union[List, Dict] = (
                data_columns if data_columns else default_data_columns
            )
            content = self.generator.json(
                data_columns=data_columns,
                num_rows=num_rows,
            )

        dataset = Dataset()
        dataset.load(content, format="json")

        storage.write_bytes(filename, dataset.export(self.extension))

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = {"content": content}
        return file_name



import os
from pathlib import Path
from random import choice
from typing import Any, Dict, Iterable, Optional, Type

from faker.providers import BaseProvider

from ...base import FileMixin, StringValue
from ...storages.base import BaseStorage
from ...storages.filesystem import FileSystemStorage
from ..helpers.inner import (
    create_inner_docx_file,
    create_inner_eml_file,
    create_inner_epub_file,
    create_inner_odt_file,
    create_inner_pdf_file,
    create_inner_rtf_file,
    create_inner_txt_file,
)
from .augmenters.base import BaseTextAugmenter
from .augmenters.nlpaug_augmenter import ContextualWordEmbeddingsAugmenter
from .extractors.base import BaseTextExtractor
from .extractors.tika_extractor import TikaTextExtractor

# Full list:
# create_inner_bin_file: N/A
# create_inner_csv_file: Not supported
# create_inner_docx_file: Supported
# create_inner_eml_file: Supported
# create_inner_epub_file: Supported
# create_inner_ico_file: Not supported
# create_inner_jpeg_file: Not supported
# create_inner_mp3_file: Not supported
# create_inner_ods_file: Not supported
# create_inner_odt_file: Supported
# create_inner_pdf_file: Supported
# create_inner_png_file: Not supported
# create_inner_pptx_file: Not supported
# create_inner_rtf_file: Supported
# create_inner_svg_file: Not supported
# create_inner_txt_file: Supported
# create_inner_webp_file: Not supported
# create_inner_xlsx_file: Not supported
# create_inner_zip_file: Not supported


__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("AugmentFileFromDirProvider",)

FILE_TYPE_TO_INNER_FUNC_MAPPING = {
    "docx": create_inner_docx_file,
    "eml": create_inner_eml_file,
    "epub": create_inner_epub_file,
    "odt": create_inner_odt_file,
    "pdf": create_inner_pdf_file,
    "rtf": create_inner_rtf_file,
    "txt": create_inner_txt_file,
}

EXTENSIONS = set(FILE_TYPE_TO_INNER_FUNC_MAPPING.keys())


class AugmentFileFromDirProvider(BaseProvider, FileMixin):
    """Augment file from given directory provider.

    Usage example:

        from faker_file.providers.augment_file_from_dir import (
            AugmentFileFromDirProvider,
        )

        file = AugmentFileFromDirProvider(None).augment_file_from_dir(
            source_dir_path="/tmp/tmp/",
        )

    Usage example with options:

        from faker_file.providers.augment_file_from_dir import (
            AugmentFileFromDirProvider,
        )

        file = AugmentFileFromDirProvider(None).augment_file_from_dir(
            source_dir_path="/tmp/tmp/",
            prefix="zzz",
            extensions={"docx", "pdf"}
        )
    """

    extension: str = ""

    def augment_file_from_dir(
        self: "AugmentFileFromDirProvider",
        source_dir_path: str,
        extensions: Iterable[str] = None,
        storage: BaseStorage = None,
        prefix: Optional[str] = None,
        wrap_chars_after: Optional[int] = None,
        text_extractor_cls: Type[BaseTextExtractor] = TikaTextExtractor,
        text_extractor_kwargs: Optional[Dict[str, Any]] = None,
        text_augmenter_cls: Type[
            BaseTextAugmenter
        ] = ContextualWordEmbeddingsAugmenter,
        text_augmenter_kwargs: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> StringValue:
        """Augment a random file from given directory.

        :param source_dir_path: Source files directory.
        :param extensions: Allowed extensions.
        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param prefix: File name prefix.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param text_extractor_cls: Text extractor class.
        :param text_extractor_kwargs: Text extractor kwargs.
        :param text_augmenter_cls: Text augmenter class.
        :param text_augmenter_kwargs: Text augmenter kwargs.
        :return: Relative path (from root directory) of the generated file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        if extensions is None:
            extensions = EXTENSIONS

        # Specific
        source_file_choices = [
            os.path.join(source_dir_path, _f)
            for _f in os.listdir(source_dir_path)
            if (
                os.path.isfile(os.path.join(source_dir_path, _f))
                and os.path.splitext(_f)[1][1:] in extensions
            )
        ]
        source_file_path = choice(source_file_choices)
        source_file = Path(source_file_path)

        if text_extractor_cls is None:
            text_extractor_cls = TikaTextExtractor

        if not text_extractor_kwargs:
            text_extractor_kwargs = {}

        text_extractor = text_extractor_cls(**text_extractor_kwargs)
        extracted_content = text_extractor.extract(source_file)
        file_type = source_file.suffix[1:]

        if text_augmenter_cls is None:
            text_augmenter_cls = ContextualWordEmbeddingsAugmenter

        if not text_augmenter_kwargs:
            text_augmenter_kwargs = {}

        text_augmenter = text_augmenter_cls(**text_augmenter_kwargs)
        content = text_augmenter.augment(extracted_content)

        return FILE_TYPE_TO_INNER_FUNC_MAPPING[file_type](
            storage=storage,
            prefix=prefix,
            content=content,
            wrap_chars_after=wrap_chars_after,
        )
